% Jacobian_RT_rodrigues 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function J=Jacobian_RT_rodrigues(wx,wy,wz)

theta = sqrt(wx*wx+wy*wy+wz*wz);

a = cos(theta);
b = (1-cos(theta))/(theta^2);
c = sin(theta)/theta;

%rotation matrix using rodrigues parameters
% R11 = a + b*(wx^2);
% R12 = b*wx*wy - c*wz;
% R13 = b*wx*wz + c*wy;
% 
% R21 = b*wx*wy + c*wz;
% R22 = a + b*(wy^2);
% R23 = b*wy*wz - c*wx;
% 
% R31 = b*wx*wz - c*wy;
% R32 = b*wz*wy + c*wx;
% R33 = a + b*(wz^2);

da = (-sin(theta)/theta);
db = sin(theta)/(theta^3) - 2*(1-cos(theta))/(theta^4);
dc = cos(theta)/(theta^2) - sin(theta)/(theta^3);

R11wx = da*wx + db*(wx^3) + 2*b*wx;
R11wy = da*wy + db*(wx^2)*wy;
R11wz = da*wz + db*(wx^2)*wz;

R12wx = db*(wx^2)*wy + b*wy - dc*wx*wz;
R12wy = db*wx*(wy^2) + b*wx - dc*wz*wy;
R12wz = db*wx*wy*wz - dc*(wz^2) - c;

R13wx = db*(wx^2)*wz + b*wz + dc*wx*wy;
R13wy = db*wx*wy*wz + dc*(wy^2) + c;
R13wz = db *wx*(wz^2) +b*wx+dc*wz*wy;

R21wx = db*(wx^2)*wy + b*wy +dc*wx*wz;
R21wy = db*(wy^2)*wx + b*wx + dc*wy*wz;
R21wz = db*wx*wy*wz + dc*(wz^2) + c;

R22wx = da*wx + db*wx*(wy^2);
R22wy = da*wy + db*(wy^3) + 2*b*wy;
R22wz = da*wz + db*wz*(wy^2);

R23wx = db*wx*wy*wz - dc*(wx^2) - c;
R23wy = db*(wy^2)*wz + b*wz - dc*wy*wx;
R23wz = db*wy*(wz^2) + b*wy - dc*wx*wz;

R31wx = db*(wx^2)*wz + b*wz - dc*wx*wy;
R31wy = db*wx*wy*wz - dc*(wy^2) -c;
R31wz = db*wx*(wz^2) +b*wx - dc*wy*wz;

R32wx = db*wx*wy*wz + dc*(wx^2) +c;
R32wy = db*(wy^2)*wz + b*wz + dc*wx*wy;
R32wz = db*wy*(wz^2) + b*wy + dc*wx*wz;

R33wx = da*wx + db*wx*(wz^2);
R33wy = da*wy + db*wy*(wz^2);
R33wz = da*wz + db*(wz^3) + 2*b*wz;


%Build the jacobian
J=zeros(12,6);
J=[R11wx R11wy R11wz 0 0 0;
   R12wx R12wy R12wz 0 0 0;
   R13wx R13wy R13wz 0 0 0;
   R21wx R21wy R21wz 0 0 0;
   R22wx R22wy R22wz 0 0 0;
   R23wx R23wy R23wz 0 0 0;
   R31wx R31wy R31wz 0 0 0;
   R32wx R32wy R32wz 0 0 0;
   R33wx R33wy R33wz 0 0 0;
     0     0     0   1 0 0;
     0     0     0   0 1 0;
     0     0     0   0 0 1];



