% compute_covariances 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/


function [covariance_uv,covariance_xyz,covariance_RT,Juv,Jxyz,JRT]=compute_covariances(X,Xcam,A,rotation,transl,covariance_Pose,covariance_sensor)

xworld=X(1); yworld=X(2); zworld=X(3);
xcam=Xcam(1); ycam=Xcam(2); zcam=Xcam(3);
wx=rotation(1); wy=rotation(2); wz=rotation(3);

%covariance rotation and translation
JRT=Jacobian_RT_rodrigues(wx,wy,wz);
covariance_RT=JRT*covariance_Pose*JRT';

%covariance xyz
Jxyz=[xworld,yworld,zworld,0,0,0,0,0,0,1,0,0;
    0,0,0,xworld,yworld,zworld,0,0,0,0,1,0;
    0,0,0,0,0,0,xworld,yworld,zworld,0,0,1];
covariance_xyz=Jxyz*covariance_RT*Jxyz';

%covariance uv
intrinsec=[A(1,1),A(2,2),A(1,3),A(2,3)];
Juv=Jacobian_uv(A,xcam,ycam,zcam);
covariance_uv=Juv*covariance_xyz*Juv';

%we add the covariance of the sensor
covariance_uv=covariance_uv+covariance_sensor;


