% error_best_pose 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/


function [R2,T2,error_repr2_true,error_repr2_est,error_rot2,error_trans2,percentage_outliers,Idx_matching3d_2d,Idx_outliers]=error_best_pose(tree,SENSOR_NOISE,A,Xw,U,Pose_ini,Xw_no_occluded,Ureal,Rtrue,Ttrue,draw_results)


%Initial projection
[Uproj_ini,R_ini,T_ini]=project_points2(A,Pose_ini,Xw);

%pose with minimum error
err=tree.error;
p=find(err);
[min_err,p2]=min(err(p));
best_node=p(p2);

Pose=tree.pose(best_node,:);
[Uproj,R,T]=project_points2(A,Pose,Xw);

%compute inliers using this pose
[err_min,percentage_outliers,Idx_matching3d_2d,Idx_outliers]=estimate_minimum_reprojection_error2(U,Uproj,tree,best_node,SENSOR_NOISE);

%Final check if the outliers are really outliers
[percentage_outliers,Idx_matching3d_2d,Idx_outliers]=recheck_outliers(U,Uproj,Idx_matching3d_2d,Idx_outliers,SENSOR_NOISE);

%refine pose using EPnP (Moreno et al. ICCV 2007)
Xw_match=Xw(Idx_matching3d_2d(:,1),:);
U_match=U(Idx_matching3d_2d(:,2),:);
[R2,T2,Xc,best_solution,opt]=efficient_pnp_gauss(Xw_match,U_match,A);


%compute relative error rotation and translation
Utrue=Ureal;
[error_repr_true,error_repr_est,error_rot,error_trans]=compute_all_errors(Xw_match,U_match,Xw_no_occluded,Utrue,R,T,A,Rtrue,Ttrue);
[error_repr2_true,error_repr2_est,error_rot2,error_trans2]=compute_all_errors(Xw_match,U_match,Xw_no_occluded,Utrue,R2,T2,A,Rtrue,Ttrue);

%do not update if the EPnP algorithm does not improve
if 1.5*error_repr_est<error_repr2_est | ( error_rot<error_rot2 & error_trans<error_trans2) 
    R2=R; T2=T;
    error_repr2_true=error_repr_true;
    error_repr2_est=error_repr_est;
    error_rot2=error_rot;
    error_trans2=error_trans;
end
    

