% find_points_low_mah_eucl_distance 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function idx_points_close=find_points_low_mah_eucl_distance(U,Ui,covariance,threshold_mah,threshold_eucl)

n=size(U,1);

dist_eucl=sqrt(sum((U-repmat(Ui,size(U,1),1)).^2,2));
dist_eucl=dist_eucl';

%find dist_max according to the covariance
d=largest_eigenvalue(covariance);
max_d=threshold_mah*sqrt(d);

%points within threshold_eucl and 2max_d distance: I only need to compute
%mahalanobis for these points
p=find(dist_eucl<max_d & dist_eucl>threshold_eucl);
dist_mah=ones(1,n)*10;

for j=p
    uj=U(j,:);
    dist_mah(j)=mahalanobis_distance(Ui,uj,covariance);
    %dist_mah(j)=mahalanobis_distance_cpp(Ui,uj,covariance); %<------------
end

idx_points_close=find(dist_mah<=threshold_mah | dist_eucl<=threshold_eucl);
    