% project_with_variance 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/


function [Uproj,covariance_uv,covariance_xyz,covariance_RT,Juv,Jxyz,JRT]=project_with_variance(A,Pose_ini,Sigma_ini,covariance_sensor,Xw,U)


wx=Pose_ini(1); wy=Pose_ini(2); wz=Pose_ini(3);
Tx=Pose_ini(4); Ty=Pose_ini(5); Tz=Pose_ini(6);
Rini=rodrigues2mat(wx,wy,wz);
Tini=[Tx,Ty,Tz]';

%project the 3d points onto the image
npoints=size(Xw,1);
Xcam=Rini*Xw'+repmat(Tini,[1,npoints]);
Uproj_h=A*Xcam;
Uproj(1,:)=Uproj_h(1,:)./Uproj_h(3,:);
Uproj(2,:)=Uproj_h(2,:)./Uproj_h(3,:);
Uproj=Uproj';

%compute variance of each 3D point when it is projected onto the image
rotation_ini=[wx,wy,wz];
covariance_pose=Sigma_ini;

%compute covariances
covariance_uv=zeros(2,2,npoints);
covariance_xyz=zeros(3,3,npoints);
covariance_RT=zeros(12,12,npoints);

for i=1:npoints
    Xi_world=Xw(i,:); %vectors in columns!!!!!!!!!!
    Xi_cam=Xcam(:,i); %vectors in rows!!!!!!!!!!
    Ui=Uproj(i,:);
    [cov_uv,cov_xyz,cov_RT,Juv,Jxyz,JRT]=compute_covariances(Xi_world,Xi_cam,A,rotation_ini,Tini,covariance_pose,covariance_sensor);
    
    covariance_uv(:,:,i)=cov_uv;
    covariance_xyz(:,:,i)=cov_xyz;
    covariance_RT(:,:,i)=cov_RT;
end






