% recheck_outliers
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/


function [percentage_outliers,Idx_matching3d_2d_out,Idx_outliers_out]=recheck_outliers(U,Uproj,Idx_matching3d_2d,Idx_outliers,SENSOR_NOISE)

noutliers=size(Idx_outliers,1);

if noutliers==0
    Idx_matching3d_2d_out=Idx_matching3d_2d;
    Idx_outliers_out=Idx_outliers;
    percentage_outliers=0;
else

    %1.-available candidates----------------------
    M=size(Uproj,1);
    N=size(U,1);
    original_idx3d=[1:M];
    original_idx2d=[1:N];

    idx3d_m=Idx_matching3d_2d(:,1);%matched points
    idx2d_m=Idx_matching3d_2d(:,2);

    idx3d_a=remove_from_list(original_idx3d,idx3d_m);%available points
    idx3d_a=idx3d_a(find(idx3d_a)); %remove the zeros from the list
    idx2d_a=remove_from_list(original_idx2d,idx2d_m);
    idx2d_a=idx2d_a(find(idx2d_a));

    Uproj2=Uproj(idx3d_a,:);
    U2=U(idx2d_a,:);

    %2.-------------------------------
    M2=size(Uproj2,1);
    N2=size(U2,1);

    Uproj_big_x=repmat(Uproj2(:,1),[1,N2]);
    Uproj_big_y=repmat(Uproj2(:,2),[1,N2]);

    U_big_x=repmat(U2(:,1)',[M2,1]);
    U_big_y=repmat(U2(:,2)',[M2,1]);

    Dx=(Uproj_big_x-U_big_x).^2;
    Dy=(Uproj_big_y-U_big_y).^2;
    D=sqrt(Dx+Dy);


    %I will first assign the point with minimum distance
    M3=M2; N3=N2;
    if N2==0
        Idx_matching3d_2d_new=[];
        Idx_outliers=idx3d_a;
    else
        Idx_matching3d_2d_new=[];
        Idx_outliers=[];
        for i=1:M3
            [minD,minpos]=min(D(:));
            %row and column of minpos. Remove row and column
            res=mod(minpos,M3);
            if res==0
                col=minpos/M3;
            else
                col=(minpos-res)/M3+1;
            end
            row=minpos-(col-1)*M3;
            D(row,:)=ones(1,N3)*1e5; %value sufficiently large
            D(:,col)=ones(M2,1)*1e5;

            %Update list of correspondences
            if minD<SENSOR_NOISE
                Idx_matching3d_2d_new=[Idx_matching3d_2d_new;
                    idx3d_a(row),idx2d_a(col)];
                idx3d_a(row)=0; %this points will not be longer available
            else
                %all 3d points available are outliers
                p=find(idx3d_a);
                Idx_outliers=[Idx_outliers,idx3d_a(p)];
                break;
            end
        end
    end

    %2.-Merge all-------------------------------------
    Idx_matching3d_2d_out=[Idx_matching3d_2d;Idx_matching3d_2d_new];
    Idx_outliers_out=Idx_outliers;
    percentage_outliers=100*size(Idx_outliers,2)/M;
end




