% generate_poses_sphere 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function Poses=generate_poses_sphere(nsamples1,nsamples2,nsamples_rotation,rad1,rad2,rad3,Xw,A)

%1.-Define the sample positions within the spheres
if nsamples1==1
    x1=[0 0 0];
else
    x1=sample_sphere(nsamples1,rad1);
end
x2=sample_empty_sphere(nsamples2,rad2,rad3);

if nsamples_rotation==1
    samples_rotation=[0];
else
    inc=floor(360/nsamples_rotation);
    samples_rotation=[0:inc:360];
end

%2.-Compute all poses, for all possible combinations
numposes=nsamples1*nsamples2*nsamples_rotation;
Poses=zeros(numposes,6);
cont=1;
for i=1:nsamples1
    for j=1:nsamples2
        p1=x1(i,:)';
        p2=x2(j,:)';
        z=p1-p2;
        z=z/norm(z);

        if z(1)==0  x=[1 0 0]';
        elseif z(2)==0  x=[0 1 0]';
        else
            x=[1,-z(1)/z(2),0]';
            x=x/norm(x);
        end
        y=cross(z,x); y=y/norm(y);
        if y(3)<0 %make sure ycam points up
            y=y*(-1);
            x=x*(-1);
        end
        R=inv([x,y,z]);
        T=-R*p2;

        %rotate around z
        for angl=samples_rotation
            angle=angl*pi/180;
            Rz=[cos(angle) -sin(angle) 0;
                sin(angle) cos(angle) 0;
                0 0 1];
            xrot=inv(R)*Rz*R*x;
            yrot=inv(R)*Rz*R*y;
            zrot=inv(R)*Rz*R*z;

            Rrot=inv([xrot,yrot,zrot]);
            rrot=rodrigues(Rrot); %exponential map form

            %project 3d points onto the image
            Up=project_points(A,Rrot,T,Xw);
            %check number of points within the image
            [Upin,percentage_inliers]=check_inliers(Up,640,480);

            if percentage_inliers>0.5
                T=-Rrot*p2;
                Poses(cont,:)=[rrot',T'];
                cont=cont+1;
                if mod(cont,300)==0 fprintf('nposes:%d of %d, n1:%d, n2:%d\n',cont,numposes,i,j); end
            end
        end
    end
end
nonzero=find(Poses(:,1)~=0 & Poses(:,2)~=0 & Poses(:,3)~=0);
Poses=Poses(nonzero,:);
figure;  hold on;
plot3(Xw(:,1),Xw(:,2),Xw(:,3),'.','color',[1 0 0],'markersize',15); 
plot3(x2(:,1),x2(:,2),x2(:,3),'.','color',[0 0 1]);
